local addonInfo, addonTable = ...
if not addonTable.KBB then addonTable.KBB = {} end
local KBB = addonTable.KBB
local KBB_Defaults = KBB.defaults

local KBBDebug = KBB.debug


KBB.KBB_Bar = {}
local KBB_Bar = KBB.KBB_Bar
KBB_Bar.__index = KBB_Bar

if not KBB.context then
	KBB.context = UI.CreateContext("KBB_Context")
	KBB.context:SetAllPoints(UIParent)
	KBB.context:SetLayer(1)
end

local function OnRightClick(self)
	KBBDebug("bar frame right click")
-- 	if not self.parentBar then return end
	if (self.parentBar.parentGroup and self.parentBar.parentGroup.settings.cancelClick) then
		Command.Buff.Cancel(self.parentBar.buffInfo.buffId)
		KBB.RescanBuffs(self.parentBar.parentGroup)
	end
end

local function OnMouseIn(bar)
-- 	    if KBB.characterDB.settings.useNativeTooltip then
-- 		    local u = Inspect.Unit.Lookup(bar.parentGroup.settings.unit)
-- 		    Command.Tooltip(u, bar.buffInfo.buffId)
-- 	    else
-- 		    KBB.showBuffTooltip(bar.buffInfo)
-- 	    end

		bar:showTooltip()

--  	Library.KLH.tdump(bar.buffInfo)
--  	Command.Tooltip(bar.buffInfo.ability)
end

local function OnMouseOut(bar)
		if KBB.characterDB and KBB.characterDB.settings.useNativeTooltip then
			Command.Tooltip(nil)
		else
			KBB.hideBuffTooltip()
		end
-- 		bar:hideTooltip()
-- 	Command.Tooltip(nil)
end

function KBB_Bar.new()
	local bar = {}
	bar.container = UI.CreateFrame("Frame", "KBB_Bar", KBB.context)
	bar.frame = UI.CreateFrame("Frame", "Frame", bar.container)
	bar.frame:SetLayer(-2)
	bar.frame:SetWidth(KBB_Defaults.group.barWidth)
	bar.frame:SetHeight(KBB_Defaults.group.barHeight)
	bar.frame.parentBar = bar

	bar.solid = UI.CreateFrame("Frame", "Solid", bar.container)
	bar.solid:SetLayer(-1)
	bar.solid:SetPoint("TOPLEFT", bar.frame, "TOPLEFT")
	bar.solid:SetPoint("BOTTOMLEFT", bar.frame, "BOTTOMLEFT")
	bar.solid:SetWidth(bar.frame:GetWidth())

	local fColor = KBB_Defaults.group.fontColor

	bar.text = UI.CreateFrame("Text", "Text", bar.container)
	bar.text:SetFontSize(KBB_Defaults.group.textFontSize)
	bar.text:SetFontColor(fColor.r/255, fColor.g / 255, fColor.b / 255)

	bar.timer = UI.CreateFrame("Text", "Timer", bar.container)
	bar.timer:SetFontSize(KBB_Defaults.group.timerFontSize)
	bar.timer:SetFontColor(fColor.r / 255, fColor.g / 255, fColor.b / 255)

	bar.icon = UI.CreateFrame("Texture", "Icon", bar.container)
	bar.icon:SetHeight(bar.frame:GetHeight())
	bar.icon:SetWidth(bar.frame:GetHeight())
	bar.icon.parentBar = bar;
	
	bar.stack = UI.CreateFrame("Text", "Stack", bar.container)
	bar.stack:SetFontSize(KBB_Defaults.group.stackFontSize)
	bar.stack:SetFontColor(fColor.r / 255, fColor.g / 255, fColor.b / 255)

--	bar.text:SetPoint("RIGHT", bar.stack, "LEFT")
	if KBB.debugMode then
		bar.text:SetBackgroundColor(1, 1, 1, 0.5)
		bar.stack:SetBackgroundColor(0, 0, 0, 0.7)
	end
	bar.container:SetVisible(false)

	bar.frame.Event.RightDown = OnRightClick
    bar.frame.Event.MouseIn = function() OnMouseIn(bar) end
    bar.frame.Event.MouseOut = OnMouseOut
    bar.icon.Event.RightDown = OnRightClick
    bar.icon.Event.MouseIn = function() OnMouseIn(bar) end
    bar.icon.Event.MouseOut = OnMouseOut

	return setmetatable(bar, KBB_Bar)
end

function KBB_Bar:hideTooltip()
    local left = self.frame:GetLeft()
	local right = self.frame:GetRight()
	local top = self.frame:GetTop()
	local bottom = self.frame:GetBottom()
	
	local mouse = Inspect.Mouse()
	
	if mouse.x >= left and mouse.x <= right and
		mouse.y >= top and mouse.y <= bottom then
-- 			KBB.hideBuffTooltip()
			if KBB.characterDB and KBB.characterDB.settings.useNativeTooltip then
				Command.Tooltip(nil)
			else
				KBB.hideBuffTooltip()
			end
	end
end
function KBB_Bar:showTooltip()
    local left = self.frame:GetLeft()
	local right = self.frame:GetRight()
	local top = self.frame:GetTop()
	local bottom = self.frame:GetBottom()
	
	local mouse = Inspect.Mouse()
	
	if mouse.x >= left and mouse.x <= right and
		mouse.y >= top and mouse.y <= bottom then
		if KBB.characterDB and KBB.characterDB.settings.useNativeTooltip then
		    local u = Inspect.Unit.Lookup(self.parentGroup.unitID)
		    Command.Tooltip(u, self.buffInfo.buffId)
	    else
		    KBB.showBuffTooltip(self.buffInfo)
	    end
		--
-- 			KBB.showBuffTooltip(self.buffInfo)
	end
end

function KBB_Bar:reset()
	
	self:hideTooltip()	
	self.parentGroup = nil
	
	self.frame:SetWidth(KBB_Defaults.group.barWidth)
	self.frame:SetHeight(KBB_Defaults.group.barHeight)

	self.buffInfo = {}
	self.container:SetVisible(false)
end

function KBB_Bar:setBuffInfo(buffId, buffInfo)
	self.buffInfo = {}
	self.buffInfo.buffId = buffId
	for k, v in pairs(buffInfo) do
		if (k == "duration") then
			self.buffInfo.duration = math.ceil(v)
		else
			self.buffInfo[k] = v
	    end
	end
end


function KBB_Bar:setBarTexts()
	local bInfo = self.buffInfo
	KBBDebug("setting up texts for bar of " .. bInfo.name)

	self.text:SetText(bInfo.name)

	KBBDebug(self.text:GetText())

	if (bInfo.stack) then
		self.stack:SetText("x" .. tostring(bInfo.stack))
	else
		self.stack:SetText("")
	end

	if (bInfo.duration and bInfo.duration > 0) then
		self.timer:SetText(Library.KLH.timeString(bInfo.remaining))
	else
		self.timer:SetText("")
	end

end

function KBB_Bar:setBarTextsAndIcon()
	local bInfo = self.buffInfo
	self:setBarTexts()
	self.icon:SetTexture("Rift", bInfo.icon)
end

function KBB_Bar:fitText(stackMargin)
	local info = self.buffInfo
	if not info then return end
	local stackMargin = stackMargin or 0
	if (info.stack) then
		self.text:SetWidth(self.stack:GetLeft() - self.text:GetLeft() - stackMargin)
	else
		if (info.duration) then
	   		self.text:SetWidth(self.frame:GetWidth() - self.timer:GetWidth() - stackMargin)
	   	else
	   		self.text:SetWidth(self.frame:GetWidth())
	   	end
	end
end

-- function KBB_Bar:updateTimer(reask)
-- 	if not self.parentGroup then return end
-- 	local buffInfo = self.buffInfo
-- 	local buffdetail
-- 	if (buffInfo.duration and buffInfo.duration > 0) then
-- 		if (reask) then	
-- 			buffInfo.remaining = 0
-- 			buffdetail = Inspect.Buff.Detail(self.parentGroup.settings.unit, self.buffInfo.buffId)
-- 			if buffdetail then buffInfo.remaining = buffdetail.remaining or 0 end
-- 		else
-- 			buffInfo.remaining = buffInfo.remaining - KBB.throttle 
-- 			if (buffInfo.remaining < 0) then  buffInfo.remaining = 0 end
-- 		end
-- 			self.timer:SetText(Library.KLH.timeString(buffInfo.remaining))
-- 	--		self.solid:SetPoint("BOTTOMRIGHT", self.frame, "BOTTOMRIGHT", (buffInfo.remaining/buffInfo.duration - 1)*self.frame:GetWidth(), 0)
-- 			self.solid:SetWidth(buffInfo.remaining / buffInfo.duration * self.frame:GetWidth())
-- 	end
-- end

function KBB_Bar:update(buffDetail)
	if not self.parentGroup then return end
	local buffInfo = self.buffInfo
	local oldrem = buffInfo.remaining
	if buffInfo.duration then
		if buffDetail then
			buffInfo.remaining = buffDetail.remaining -- update remaining time
		else
			buffInfo.remaining = buffInfo.remaining - KBB.throttle -- subtract update interval
			if (buffInfo.remaining < 0) then  buffInfo.remaining = 0 end -- can't be negative
		end
		if (oldrem ~= buffInfo.remaining) then -- if remaining time changed
			self.timer:SetText(Library.KLH.timeString(buffInfo.remaining)) -- update timer text
			self.solid:SetWidth(buffInfo.remaining / buffInfo.duration * self.frame:GetWidth()) --update solid 
		end
	end
-- 	self.solid:SetLayer(-1)
end

function KBB_Bar:updateLayout(layout)
	self.text:SetPoint(layout.textPos.anchorFrom, self.frame, layout.textPos.anchorTo,
		layout.textPos.x, layout.textPos.y)
	self.timer:SetPoint(layout.timerPos.anchorFrom, self.frame, layout.timerPos.anchorTo,
		layout.timerPos.x, layout.timerPos.y)
	self.icon:SetPoint(layout.iconPos.anchorFrom, self.frame, layout.iconPos.anchorTo,
		layout.iconPos.x, layout.iconPos.y)
--	print(layout.stackPos.anchorFrom, layout.stackPos.anchorTo,
--		layout.stackPos.x, layout.stackPos.y )
	self.stack:SetPoint(layout.stackPos.anchorFrom, self.timer, layout.stackPos.anchorTo,
		layout.stackPos.x, layout.stackPos.y)
	self.solid:SetWidth(self.frame:GetWidth())

--	self.text:SetPoint("CENTERRIGHT", self.stack, "CENTERLEFT")
end

function KBB_Bar:updateColor()
	if not self.parentGroup then return end
	local color = {}
	local parentGroup = self.parentGroup
	if (parentGroup.settings.watchType == "buff") then
		color = parentGroup.settings.buffColor
	elseif (parentGroup.settings.watchType == "debuff") then
		color = parentGroup.settings.debuffColor
	end
	self.solid:SetBackgroundColor(color.r / 255, color.g / 255, color.b / 255)
	if (not parentGroup.settings.backgroundUsesSameColor) then
		color = parentGroup.settings.backgroundColor
	end
	self.frame:SetBackgroundColor(color.r / 255, color.g / 255, color.b / 255,
								parentGroup.settings.backgroundAlpha / 255)
end

function KBB_Bar:updateFontColor()
	if not self.parentGroup then return end
	local parentGroup = self.parentGroup
	local color = {}
	if (parentGroup.settings.watchType == "buff") then
		color = parentGroup.settings.fontColor
	elseif (parentGroup.settings.watchType == "debuff") then
		color = parentGroup.settings.debuffFontColor
	end
	self.text:SetFontColor(color.r/255, color.g/255, color.b/255)
	self.timer:SetFontColor(color.r/255, color.g/255, color.b/255)
	self.text:SetFontColor(color.r/255, color.g/255, color.b/255)
end

function KBB_Bar:updateBackgroundAlpha()
	if not self.parentGroup then return end
	local alpha = self.parentGroup.settings.backgroundAlpha
--	print("alpha before: "..self.frame:GetAlpha())
--	print("new alpha: ".. alpha)
--	self.frame:SetAlpha(alpha)
--	print("alpha after: " .. self.frame:GetAlpha())

	local r,g,b = self.frame:GetBackgroundColor()
	self.frame:SetBackgroundColor(r, g, b, alpha / 255)
end


function KBB_Bar:setWidth(w)
	local oldWidth = self.frame:GetWidth()
	local oldSolid = self.solid:GetWidth()
	self.frame:SetWidth(w)
	self.solid:SetWidth(oldSolid * w / oldWidth)
end


function KBB_Bar:setHeight(w)
	self.frame:SetHeight(w)
	self.icon:SetHeight(w)
	self.icon:SetWidth(w)
end

function KBB_Bar:setFontSize(s)
	local layout = self.parentGroup.settings.barLayout
	local offset = layout.stackPos.x * s / math.max(self.stack:GetFontSize(), self.timer:GetFontSize())
--	print ("----")
--	Library.KLH.tdump(self.parentGroup.settings.barLayout)
	layout.stackPos.x = offset
--	print("----")
--	Library.KLH.tdump(self.parentGroup.settings.barLayout)
	self.text:SetFontSize(s)
	self.stack:SetFontSize(s)
	self.timer:SetFontSize(s)
end

function KBB_Bar:setTextFontSize(s)
	self.text:SetFontSize(s)
end

function KBB_Bar:setStackFontSize(s)
	self.stack:SetFontSize(s)
end

function KBB_Bar:setTimerFontSize(s)
	local layout = self.parentGroup.settings.barLayout
	local offset = layout.stackPos.x * s / math.max(self.stack:GetFontSize(), self.timer:GetFontSize())
	layout.stackPos.x = offset

	self.timer:SetFontSize(s)
end

function KBB_Bar:setFontColor(r, g, b)
	self.text:SetFontColor(r / 255, g / 255, b / 255)
	self.timer:SetFontColor(r / 255, g / 255, b / 255)
	self.stack:SetFontColor(r / 255, g / 255, b / 255)
end

function KBB_Bar.__lt (b1, b2)
	local r1 = b1.buffInfo.remaining
	local r2 = b2.buffInfo.remaining
	if (r1 == nil) then
		return false
	elseif (r2 == nil) then
		return true
	else return r1 < r2 end
end
